@@ -11,7 +11,6 @@
 #include <glade/glade.h>
 #include <X11/Xatom.h>
 
-#include "gnome-theme-info.h"
 #include "wm-common.h"
 #include "capplet-util.h"
 #include "eggcellrendererkeys.h"
@@ -102,6 +101,7 @@ static const KeyListEntry metacity_key_list[] =
   { "/apps/metacity/global_keybindings/switch_windows",            ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/switch_panels",             ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/cycle_windows",             ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/global_keybindings/cycle_group",               ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/cycle_panels",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/show_desktop",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/switch_to_workspace_1",     N_WORKSPACES_GT, 1 },
@@ -170,7 +170,7 @@ create_dialog (void)
 {
   GladeXML *dialog;
 
-  dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-keybinding-properties.glade", "gnome-keybinding-dialog", NULL);
+  dialog = glade_xml_new (GNOMECC_GLADE_DIR "/gnome-keybinding-properties.glade", "gnome-keybinding-dialog", NULL);
 
   return dialog;
 }
@@ -182,7 +182,9 @@ binding_name (guint                   keyval,
               gboolean                translate)
 {
   if (keyval != 0 || keycode != 0)
-    return egg_virtual_accelerator_name (keyval, keycode, mask);
+    return translate ?
+	egg_virtual_accelerator_label (keyval, keycode, mask) :
+	egg_virtual_accelerator_name (keyval, keycode, mask);
   else
     return translate ? g_strdup (_("Disabled")) : g_strdup ("disabled");
 }
@@ -297,12 +299,8 @@ keyentry_sort_func (GtkTreeModel *model,
 {
   KeyEntry *key_entry_a;
   KeyEntry *key_entry_b;
-  char *name_a;
-  char *name_b;
   int retval;
 
-  /* mmmmm, super-slow. */
-
   key_entry_a = NULL;
   gtk_tree_model_get (model, a,
                       KEYENTRY_COLUMN, &key_entry_a,
@@ -313,44 +311,52 @@ keyentry_sort_func (GtkTreeModel *model,
                       KEYENTRY_COLUMN, &key_entry_b,
                       -1);
 
-  
-  if (key_entry_a != NULL)
-    name_a = binding_name (key_entry_a->keyval,
-		    	   key_entry_a->keycode,
-                           key_entry_a->mask,
-                           TRUE);
-  else
-    name_a = NULL;
+  if (key_entry_a && key_entry_b)
+    {
+      if ((key_entry_a->keyval || key_entry_a->keycode) &&
+          (key_entry_b->keyval || key_entry_b->keycode))
+        {
+          gchar *name_a, *name_b;
 
-  if (key_entry_b != NULL)
-    name_b = binding_name (key_entry_b->keyval,
-		    	   key_entry_b->keycode,
-                           key_entry_b->mask,
-                           TRUE);
-  else
-    name_b = NULL;
+          name_a = binding_name (key_entry_a->keyval,
+                                 key_entry_a->keycode,
+                                 key_entry_a->mask,
+                                 TRUE);
 
-  if (name_a && name_b)
-    retval = g_utf8_collate (name_a, name_b);
-  else if (name_a)
-    retval = 1;
-  else if (name_b)
+          name_b = binding_name (key_entry_b->keyval,
+                                 key_entry_b->keycode,
+                                 key_entry_b->mask,
+                                 TRUE);
+
+          retval = g_utf8_collate (name_a, name_b);
+
+          g_free (name_a);
+          g_free (name_b);
+        }
+      else if (key_entry_a->keyval || key_entry_a->keycode)
+        retval = -1;
+      else if (key_entry_b->keyval || key_entry_b->keycode)
+        retval = 1;
+      else
+        retval = 0;
+    }
+  else if (key_entry_a)
     retval = -1;
+  else if (key_entry_b)
+    retval = 1;
   else
     retval = 0;
 
-  g_free (name_a);
-  g_free (name_b);
-
   return retval;
 }
 
 static void
-clear_old_model (GladeXML  *dialog,
-                 GtkWidget *tree_view)
+clear_old_model (GladeXML  *dialog)
 {
+  GtkWidget *tree_view;
   GtkTreeModel *model;
 
+  tree_view = WID ("shortcut_treeview");
   model = get_real_model (GTK_TREE_VIEW (tree_view));
 
   if (model == NULL)
@@ -362,9 +368,8 @@ clear_old_model (GladeXML  *dialog,
   
       sort_model = gtk_tree_model_sort_new_with_model (model);
 
-      /* N_COLUMNS is just a place to stick the extra sort function */
       gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (sort_model),
-                                   	   N_COLUMNS,
+                                       KEYENTRY_COLUMN,
                                        keyentry_sort_func,
                                        NULL, NULL);
   
@@ -558,7 +563,7 @@ append_keys_to_tree (GladeXML           *dialog,
 static void
 reload_key_entries (gpointer wm_name, GladeXML *dialog)
 {
-  clear_old_model (dialog, WID ("shortcut_treeview"));
+  clear_old_model (dialog);
   
   append_keys_to_tree (dialog, _("Desktop"), desktop_key_list);
   append_keys_to_tree (dialog, _("Sound"), sounds_key_list);
@@ -637,6 +642,9 @@ accel_edited_callback (GtkCellRendererText   *cell,
   if (key_entry == NULL)
     return;
 
+  /* CapsLock isn't supported as a keybinding modifier, so keep it from confusing us */
+  mask &= ~EGG_VIRTUAL_LOCK_MASK;
+
   model = get_real_model (view);
   tmp_key.model  = model;
   tmp_key.keyval = keyval;
@@ -656,6 +664,7 @@ accel_edited_callback (GtkCellRendererText   *cell,
     {
       if ((tmp_key.keyval >= GDK_a && tmp_key.keyval <= GDK_z)
 	   || (tmp_key.keyval >= GDK_A && tmp_key.keyval <= GDK_Z)
+	   || (tmp_key.keyval >= GDK_0 && tmp_key.keyval <= GDK_9)
 	   || (tmp_key.keyval >= GDK_kana_fullstop && tmp_key.keyval <= GDK_semivoicedsound)
 	   || (tmp_key.keyval >= GDK_Arabic_comma && tmp_key.keyval <= GDK_Arabic_sukun)
 	   || (tmp_key.keyval >= GDK_Serbian_dje && tmp_key.keyval <= GDK_Cyrillic_HARDSIGN)
@@ -664,18 +673,19 @@ accel_edited_callback (GtkCellRendererText   *cell,
 	   || (tmp_key.keyval >= GDK_Thai_kokai && tmp_key.keyval <= GDK_Thai_lekkao)
 	   || (tmp_key.keyval >= GDK_Hangul && tmp_key.keyval <= GDK_Hangul_Special)
 	   || (tmp_key.keyval >= GDK_Hangul_Kiyeog && tmp_key.keyval <= GDK_Hangul_J_YeorinHieuh)
+	   || tmp_key.keyval == GDK_space || tmp_key.keyval == GDK_Return
 	   || tmp_key.keyval == GDK_Mode_switch) {
         GtkWidget *dialog;
 	char *name;
 
-	name = egg_virtual_accelerator_name (keyval, keycode, mask);
+	name = binding_name (keyval, keycode, mask, TRUE);
 
 	dialog =
 	  gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
 			  	  GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
 				  GTK_MESSAGE_WARNING,
 				  GTK_BUTTONS_CANCEL,
-				  _("The shortcut \"%s\" cannot be used because it will become unusable to type using this key.\n"
+				  _("The shortcut \"%s\" cannot be used because it will become impossible to type using this key.\n"
 				  "Please try with a key such as Control, Alt or Shift at the same time.\n"),
 				  name);
 
@@ -697,7 +707,7 @@ accel_edited_callback (GtkCellRendererText   *cell,
       GtkWidget *dialog;
       char *name;
 
-      name = egg_virtual_accelerator_name (keyval, keycode, mask);
+      name = binding_name (keyval, keycode, mask, TRUE);
       
       dialog =
         gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
@@ -869,11 +879,18 @@ static void
 cb_dialog_response (GtkWidget *widget, gint response_id, gpointer data)
 {
 	if (response_id == GTK_RESPONSE_HELP)
-		capplet_help (GTK_WINDOW (widget),
-			      "user-guide.xml",
-			      "goscustdesk-39");
+          {
+            capplet_help (GTK_WINDOW (widget),
+                          "user-guide.xml",
+                          "goscustdesk-39");
+          }
 	else
-		gtk_main_quit ();
+          {
+            GladeXML *dialog = data;
+
+            clear_old_model (dialog);
+            gtk_main_quit ();
+          }
 }
 
 static void
@@ -923,8 +940,7 @@ setup_dialog (GladeXML *dialog)
   gtk_tree_view_column_set_resizable (column, FALSE);
 
   gtk_tree_view_append_column (GTK_TREE_VIEW (WID ("shortcut_treeview")), column);
-  /* N_COLUMNS is just a place to stick the extra sort function */
-  gtk_tree_view_column_set_sort_column_id (column, N_COLUMNS); 
+  gtk_tree_view_column_set_sort_column_id (column, KEYENTRY_COLUMN); 
   
   gconf_client_add_dir (client, "/apps/gnome_keybinding_properties", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_add_dir (client, "/apps/metacity/general", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -943,8 +959,7 @@ setup_dialog (GladeXML *dialog)
   capplet_set_icon (widget, "gnome-settings-keybindings");
   gtk_widget_show (widget);
 
-  g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK(cb_dialog_response), NULL);
-  g_signal_connect (G_OBJECT (widget), "close", gtk_main_quit, NULL);
+  g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK(cb_dialog_response), dialog);
 }
 
 int
@@ -964,8 +979,6 @@ main (int argc, char *argv[])
 		      GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 		      NULL);
 
-  gnome_theme_init (NULL);
-
   activate_settings_daemon ();
 
   dialog = create_dialog ();
@@ -974,7 +987,6 @@ main (int argc, char *argv[])
   
   gtk_main ();
 
-  clear_old_model (dialog, WID ("shortcut_treeview"));
   g_object_unref (dialog);
   g_object_unref (program);
   return 0;
